@@ -15,92 +15,61 @@
 # limitations under the License.
 #
 
-from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling.Initializer import Initializer
+
+from ai.h2o.sparkling import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
-from ai.h2o.sparkling.ml.algos.H2OTreeBasedSupervisedAlgorithm import H2OTreeBasedSupervisedAlgorithm
-from ai.h2o.sparkling.ml.models.H2ODRFMOJOModel import H2ODRFMOJOModel
-from ai.h2o.sparkling.ml.params.H2ODRFParams import H2ODRFParams
-from ai.h2o.sparkling.ml.algos.DistributionBasedH2OTrainFramePreparation import DistributionBasedH2OTrainFramePreparation
+from ai.h2o.sparkling.ml.algos.H2OTreeBasedSupervisedAlgoBase import H2OTreeBasedSupervisedAlgoBase
+from ai.h2o.sparkling.ml.params import H2ODRFParams
 
-
-class H2ODRF(H2ODRFParams, H2OTreeBasedSupervisedAlgorithm, DistributionBasedH2OTrainFramePreparation):
+class H2ODRF(H2ODRFParams, H2OTreeBasedSupervisedAlgoBase):
 
     @keyword_only
     def __init__(self,
-                 calibrationDataFrame=None,
-                 ignoredCols=None,
-                 columnsToCategorical=[],
-                 keepBinaryModels=False,
-                 withContributions=False,
-                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
-                 withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
-                 convertInvalidNumbersToNa=False,
-                 detailedPredictionCol="detailed_prediction",
-                 validationDataFrame=None,
-                 featuresCols=[],
-                 predictionCol="prediction",
-                 convertUnknownCategoricalLevelsToNa=False,
-                 splitRatio=1.0,
-                 withStageResults=False,
-                 mtries=-1,
                  binomialDoubleTrees=False,
-                 sampleRate=0.632,
-                 balanceClasses=False,
-                 classSamplingFactors=None,
-                 maxAfterBalanceSize=5.0,
-                 maxConfusionMatrixSize=20,
+                 mtries=-1,
                  ntrees=50,
                  maxDepth=20,
-                 minRows=1.0,
+                 minRows=1,
                  nbins=20,
-                 nbinsTopLevel=1024,
                  nbinsCats=1024,
-                 seed=-1,
-                 buildTreeOneNode=False,
-                 sampleRatePerClass=None,
-                 colSampleRatePerTree=1.0,
-                 colSampleRateChangePerLevel=1.0,
-                 scoreTreeInterval=0,
-                 minSplitImprovement=1.0E-5,
+                 minSplitImprovement=1e-5,
                  histogramType="AUTO",
-                 calibrateModel=False,
-                 checkConstantResponse=True,
+                 nbinsTopLevel=1 << 10,
+                 buildTreeOneNode=False,
+                 scoreTreeInterval=0,
+                 sampleRate=1.0,
+                 sampleRatePerClass=None,
+                 colSampleRateChangePerLevel=1.0,
+                 colSampleRatePerTree=1.0,
                  modelId=None,
-                 nfolds=0,
-                 keepCrossValidationModels=True,
                  keepCrossValidationPredictions=False,
                  keepCrossValidationFoldAssignment=False,
+                 parallelizeCrossValidation=True,
                  distribution="AUTO",
                  labelCol="label",
+                 foldCol=None,
                  weightCol=None,
                  offsetCol=None,
-                 foldCol=None,
-                 foldAssignment="AUTO",
-                 categoricalEncoding="AUTO",
-                 ignoreConstCols=True,
-                 scoreEachIteration=False,
+                 splitRatio=1.0,
+                 seed=-1,
+                 nfolds=0,
+                 allStringColumnsToCategorical=True,
+                 columnsToCategorical=[],
+                 predictionCol="prediction",
+                 detailedPredictionCol="detailed_prediction",
+                 withDetailedPredictionCol=False,
+                 featuresCols=[],
+                 convertUnknownCategoricalLevelsToNa=False,
+                 convertInvalidNumbersToNa=False,
+                 namedMojoOutputColumns=True,
                  stoppingRounds=0,
-                 maxRuntimeSecs=0.0,
                  stoppingMetric="AUTO",
                  stoppingTolerance=0.001,
-                 gainsliftBins=-1,
-                 customMetricFunc=None,
-                 exportCheckpointsDir=None,
-                 aucType="AUTO"):
+                 withContributions=False):
         Initializer.load_sparkling_jar()
         super(H2ODRF, self).__init__()
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.H2ODRF", self.uid)
         self._setDefaultValuesFromJava()
         kwargs = Utils.getInputKwargs(self)
-        kwargs = self._updateInitKwargs(kwargs)
-
-        if 'interactionPairs' in kwargs:
-            warn("Interaction pairs are not supported!")
         self._set(**kwargs)
-        self._transfer_params_to_java()
-
-    def _create_model(self, javaModel):
-        return H2ODRFMOJOModel(javaModel)
